local CrashSkinCheck = CrashAddon.CrashSkinCheck
local SpeedCalc = CrashAddon.SpeedCalc
local StateSetter = CrashAddon.StateSetter

//this handles how Aku Aku spawns in
local function AkuSpawnAndDie(player)
	if not CrashSkinCheck(player)
		return
	end
	
	if player.playerstate == PST_DEAD and player.crashban.akulevel > 0
		player.crashban.akulevel = 0
	end
	//this adds to the aku level and removes the shield when Crash gets one
	if player.powers[pw_shield]
		if player.powers[pw_shield] != SH_PINK
			if not player.powers[pw_invulnerability]
				player.crashban.akulevel = $ + 1
				player.crashban.akulevel = min($, 3)
				S_StartSound(player.mo, sfx_akuspn)
			end
			player.powers[pw_shield] = SH_NONE
		end
	end
	//this either spawns in Aku Aku or kills him based on aku level
	if player.crashban.akulevel <= 0 and not player.powers[pw_invulnerability]
		if player.crashban.aku and player.crashban.aku.valid
			P_RemoveMobj(player.crashban.aku)
			if not S_SoundPlaying(player.mo, sfx_akuded)
				S_StartSound(player.mo, sfx_akuded)
			end
		end
	else
		if not player.crashban.aku or not player.crashban.aku.valid
			player.crashban.aku = P_SpawnMobjFromMobj(player.mo, 0, 0, 0, 
					MT_AKUAKU)
			player.crashban.aku.target = player.mo
			if not S_SoundPlaying(player.mo, sfx_akuspn)
				S_StartSound(player.mo, sfx_akuspn)
			end
		end
		if player.powers[pw_invulnerability]
			player.crashban.akulevel = 3
		end
	end
end

//this determines what happens if Crash gets hit
local function CrashDamage(target, inflictor, source, damage, damagetype)
	if target.skin != "crashbandi" or target.player.powers[pw_carry] ==
			CR_NIGHTSMODE or target.player.powers[pw_carry] == CR_NIGHTSFALL
		return
	end
	
	if target.player.crashban.akulevel <= 0
		P_KillMobj(target, inflictor, source, damagetype)
	else
		target.player.crashban.akulevel = $ - 1
		target.player.crashban.akulevel = max($, 0)
		S_StartSound(target, sfx_altow1)
		S_StartSound(target, sfx_akuded)
		P_DoPlayerPain(target.player, source, inflictor)
	end
	return true
end

//this handles Aku Aku's behavior
local function AkuAkuThinker(mo)
	local akulevel = mo.target.player.crashban.akulevel
	//failsafe
	if akulevel == nil or akulevel <= 0 or mo.target == nil or not mo.target.valid
		P_RemoveMobj(mo)
		return
	end
	mo.scale = mo.target.scale
	mo.shadowscale = mo.scale - mo.scale / 4
	if mo.target.eflags & MFE_VERTICALFLIP
		mo.eflags = $ | MFE_VERTICALFLIP
	else
		mo.eflags = $ & ~MFE_VERTICALFLIP
	end
	//this changes Aku Aku's color based on aku level
	if akulevel > 1 and not mo.colorized
		mo.colorized = true
		mo.color = SKINCOLOR_ORANGE
	elseif akulevel <= 1 and mo.colorized
		mo.colorized = false
	end
	local invincibility = mo.target.player.powers[pw_invulnerability]
	//this makes Aku Aku orbit around the player
	if akulevel < 3
		if invincibility
			mo.target.player.crashban.akulevel = 3
			if not S_SoundPlaying(mo.target, sfx_akuspn)
				S_StartSound(mo.target, sfx_akuspn)
			end
		end
		mo.threshold = 40 * mo.scale
		A_UnidusBall(mo)
		A_FaceTarget(mo)
		mo.angle = $ + ANGLE_180
	//this gives Crash invincibility and moves Aku Aku once max level is reached
	else
		A_ForceStop(mo)
		if not invincibility
			if not mo.invin
				A_Invincibility(mo)
				mo.invin = true
			else
				mo.invin = false
				mo.target.player.crashban.akulevel = 1
				if not S_SoundPlaying(mo.target, sfx_akuded)
					S_StartSound(mo.target, sfx_akuded)
				end
			end
		elseif not mo.invin
			mo.invin = true
		end
		local akuheight
		if mo.eflags & MFE_VERTICALFLIP
			akuheight = mo.target.height / 2 * -1 + mo.target.height / 3 + 
					mo.target.height / 10
		else
			akuheight = mo.target.height  - mo.target.height / 3 - 
					mo.target.height / 10
		end
		mo.angle = mo.target.player.drawangle
		P_MoveOrigin(mo, mo.target.x + P_ReturnThrustX(mo, mo.angle, 20 * 
				mo.scale), mo.target.y + P_ReturnThrustY(mo, mo.angle, 20 * 
				mo.scale), akuheight + mo.target.z)
	end
end

//hooks
addHook("PlayerThink", AkuSpawnAndDie)
addHook("MobjDamage", CrashDamage, MT_PLAYER)
addHook("MobjThinker", AkuAkuThinker, MT_AKUAKU)