local CrashSkinCheck = CrashAddon.CrashSkinCheck
local ActionButton = CrashAddon.ActionButton
local JumpManager = CrashAddon.JumpManager
local FailVibeCheck = CrashAddon.FailVibeCheck
local CootOnGround = CrashAddon.CootOnGround

//has its own statesetter function due to the strong power flags and same state
local function StateSetter(player)
	if player.mo.state != S_PLAY_MELEE
		player.powers[pw_strong] = STR_ANIM|STR_FLOOR|STR_SPRING|STR_HEAVY|
				STR_STOMP
		player.mo.state = S_PLAY_MELEE
	end
end

local function BodySlam(player)
	if not CrashSkinCheck(player)
		return
	end
	
	//this resets everything and prevents moves if crash fails the vibe check
	if FailVibeCheck(player)
		player.crashban.bodyslam = false
		player.crashban.hover = -1
		player.crashban.faceplant = 0
		player.height = skins[player.mo.skin].height
		return
	end
	
	//mostly ported from Emmette, but with some tweaks to optimize it
	//this triggers the body slam if the right conditions are met
	if (player.pflags & PF_JUMPED and not (player.pflags & PF_STARTJUMP) or 
			player.mo.state == S_PLAY_FALL) and not (CootOnGround(player.mo) or 
			player.mo.state == S_PLAY_SPINDASH) and player.cmd.buttons & 
			ActionButton(player) and not player.crashban.slidejumped
		if not player.crashban.bodyslam
			A_ForceStop(player.mo, 0)
			S_StartSound(player.mo, sfx_cshflp)
			JumpManager(player.mo, 7 * FRACUNIT)
			player.crashban.hover = TICRATE
			player.crashban.bodyslam = true
		end
	end
	//this handles what happens during the slam
	if player.crashban.bodyslam
		player.pflags = $ | PF_FULLSTASIS
		player.height = player.spinheight
		//this makes crash hover in the air briefly
		if player.crashban.hover > 0
			if player.mo.momz == 0
				player.mo.momz = 0
			end
			StateSetter(player)
			player.crashban.hover = $ - 4
			player.crashban.hover = max($, 0)
		//this makes crash fall once he's done hovering
		elseif player.crashban.hover == 0
			JumpManager(player.mo, -40 * FRACUNIT)
			player.crashban.hover = $ - 1
		else
			if player.mo.eflags & MFE_GOOWATER or (player.mo.momz > 0 and not 
					(player.mo.eflags & MFE_VERTICALFLIP)) or (player.mo.momz < 
					0 and player.mo.eflags & MFE_VERTICALFLIP)
				player.crashban.bodyslam = false
			//this handles what happens while crash is falling
			elseif not CootOnGround(player.mo)
				StateSetter(player)
			//this spawns the attack radius for actually impacting the ground
			else
				S_StartSound(player.mo, sfx_pstop)
				local px = player.mo.x
				local py = player.mo.y
				local br = 75 * player.mo.scale
				local mo = player.mo
				P_Earthquake(player.mo, player.mo, 90 * player.mo.scale)
				searchBlockmap("objects", function(mo, found)
					if found and found.valid and found.health and 
							P_CheckSight(mo, found) and (found.z < mo.z + 
							mo.height / 2 and found.z > mo.z - mo.height / 2)
						P_DamageMobj(found, mo, mo, 1, 0)
					end
				end, mo, px-br, px+br, py-br, py+br)
				player.mo.state = S_PLAY_MELEE_LANDING
				player.crashban.faceplant = TICRATE
				player.crashban.bodyslam = false
			end
		end
	end
	//this handles what happens after crash has slammed down
	if player.crashban.faceplant > 0
		player.pflags = $ | PF_FULLSTASIS
		player.crashban.faceplant = $ - 3
	elseif player.mo.state == S_PLAY_MELEE_LANDING
		player.mo.state = S_PLAY_STND
		player.height = skins[player.mo.skin].height
	end
end

//hooks
addHook("PlayerThink", BodySlam)