local CrashSkinCheck = CrashAddon.CrashSkinCheck
local StateSetter = CrashAddon.StateSetter
local SpeedCalc = CrashAddon.SpeedCalc
local ActionButton = CrashAddon.ActionButton
local FailVibeCheck = CrashAddon.FailVibeCheck
local CootOnGround = CrashAddon.CootOnGround

local function CrawlAndSlide(player)
	if not CrashSkinCheck(player)
		return
	end
	
	//this resets everything and prevents moves if crash fails the vibe check
	if FailVibeCheck(player)
		player.normalspeed = skins[player.mo.skin].normalspeed
		player.height = skins[player.mo.skin].height
		player.crashban.slidetimer = 0
		return
	end
	
	//this resets the height and speed changes made by crawling and sliding
	if not (player.cmd.buttons & ActionButton(player)) and 
			player.crashban.slidetimer <= 0 and not player.crashban.bodyslam and 
			player.crashban.faceplant <= 0
		player.height = skins[player.mo.skin].height
		player.normalspeed = skins[player.mo.skin].normalspeed
	end
	//this handles what happens when crash hits a spring while crawling/sliding
	if CootOnGround(player.mo) and (player.cmd.buttons & ActionButton(player) or 
			player.crashban.slidetimer > 0) and player.mo.eflags & MFE_SPRUNG
		player.mo.state = S_PLAY_ROLL
		player.pflags = $ | PF_SPINNING
		player.crashban.slidetimer = 0
	end
	//this makes Crash crawl or start sliding
	if player.cmd.buttons & ActionButton(player) and CootOnGround(player.mo) and 
			player.crashban.spintimer <= 0 and not (player.pflags & PF_SPINNING)
		player.height = player.spinheight
		if player.normalspeed != 5 * FRACUNIT and SpeedCalc(player) >
				3 * player.mo.scale
			if player.crashban.slidetimer <= 0
				//this is done to nerf the slide force underwater
				local thrustspeed
				if SpeedCalc(player) > 30 * player.mo.scale
					thrustspeed = SpeedCalc(player) + 10 * player.mo.scale
				else
					thrustspeed = 40 * player.mo.scale
				end
				if player.mo.eflags & MFE_UNDERWATER or player.mo.eflags & 
						MFE_GOOWATER or P_InSpaceSector(player.mo)
					thrustspeed = $ - ($ / 3)
				end
				P_InstaThrust(player.mo, player.drawangle, thrustspeed)
				S_StartSound(player.mo, sfx_cshsld)
				player.crashban.slidetimer = TICRATE / 2
			end
		elseif player.mo.state != S_PLAY_GLIDE
			player.normalspeed = 5 * FRACUNIT
			if player.mo.momx == player.cmomx and player.mo.momy == 
					player.cmomy
				StateSetter(player.mo, S_PLAY_CLING)
			else
				StateSetter(player.mo, S_PLAY_CLIMB)
			end
		end
	else
		if player.mo.state == S_PLAY_CLING
			player.mo.state = S_PLAY_STND
		elseif player.mo.state == S_PLAY_CLIMB
			player.mo.state = S_PLAY_WALK
		end
	end
	//this handles what happens during the slide and after
	if player.crashban.slidetimer > 0
		player.pflags = $ | PF_STASIS
		if player.mo.state != S_PLAY_GLIDE
			player.powers[pw_strong] = STR_ANIM|STR_WALL|STR_PUNCH
			player.mo.state = S_PLAY_GLIDE
		end
		P_SpawnSkidDust(player, 10 * player.mo.scale)
		player.normalspeed = 5 * FRACUNIT
		player.crashban.slidetimer = $ - 1
	elseif player.mo.state == S_PLAY_GLIDE
		player.mo.state = S_PLAY_WALK
	end
end

//hooks
addHook("PlayerThink", CrawlAndSlide)